import "./DateRangeInput.css";
import { Box } from "../../box";
import { CalendarIcon, VerticalDotsIcon } from "../../icons";
import { useCallback, useImperativeHandle, useRef, useState } from "react";
import { CrossButton, IconButton } from "../../button";
import { Divider } from "../../divider";
import { Keys, augmentElement, cssModule, forwardRef, isNil, isNilOrEmpty, isNumber, mergeProps, omitProps, useAutoFocus, useControllableState, useEventCallback, useFocusWithin, useMergedRefs } from "../../shared";
import { Item } from "../../collection";
import { Menu, MenuTrigger } from "../../menu";
import { useDateInput } from "./useDateInput";
import { useFieldInputProps } from "../../field";
import { useToolbarProps } from "../../toolbar";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
import { Fragment as _Fragment } from "react/jsx-runtime";
const DateInput = forwardRef(({
  value,
  placeholder = "dd/mm/yyyy",
  required,
  validationState,
  min,
  max,
  onChange,
  onDateChange,
  autoFocus,
  disabled,
  readOnly,
  ...rest
}, ref) => {
  const inputRef = useMergedRefs(ref);
  useAutoFocus(inputRef, {
    isDisabled: !autoFocus || disabled || readOnly,
    delay: isNumber(autoFocus) ? autoFocus : undefined
  });
  const dateProps = useDateInput({
    value,
    min,
    max,
    onChange,
    onDateChange,
    forwardedRef: inputRef
  });
  return /*#__PURE__*/_jsx("input", { ...mergeProps(rest, {
      placeholder,
      className: "o-ui-date-range-input-date-input",
      type: "text",
      disabled,
      readOnly,
      "aria-required": required ? true : undefined,
      "aria-invalid": validationState === "invalid" ? true : undefined,
      ref: inputRef
    }, dateProps)
  });
});
export function InnerDateRangeInput(props) {
  const [toolbarProps] = useToolbarProps();
  const [fieldProps, isInField] = useFieldInputProps();
  const {
    startDate: startDateProp,
    endDate: endDateProp,
    defaultStartDate,
    defaultEndDate,
    placeholder,
    min,
    max,
    required,
    validationState,
    onDatesChange,
    onFocus,
    onBlur,
    presets,
    autoFocus,
    fluid,
    disabled,
    readOnly,
    active,
    focus = false,
    hover,
    name,
    as = "div",
    forwardedRef,
    ...rest
  } = mergeProps(props, omitProps(toolbarProps, ["orientation"]), fieldProps);
  const [startDate, setStartDate] = useControllableState(startDateProp, defaultStartDate, null);
  const [endDate, setEndDate] = useControllableState(endDateProp, defaultEndDate, null);
  const [hasFocus, setHasFocus] = useState(focus);
  const containerRef = useRef();
  const startDateRef = useRef();
  const endDateRef = useRef();
  useImperativeHandle(forwardedRef, () => {
    const element = containerRef.current;

    element.focus = () => {
      var _startDateRef$current;

      (_startDateRef$current = startDateRef.current) === null || _startDateRef$current === void 0 ? void 0 : _startDateRef$current.focus();
    };

    return element;
  });
  const applyDates = useCallback((event, newStartDate, newEndDate) => {
    if (startDate !== newStartDate || endDate !== newEndDate) {
      if (!isNil(onDatesChange)) {
        onDatesChange(event, newStartDate, newEndDate);
      }

      setStartDate(newStartDate);
      setEndDate(newEndDate);
    }
  }, [onDatesChange, startDate, setStartDate, endDate, setEndDate]);
  const handleStartDateChange = useEventCallback((event, newDate) => {
    if (!isNil(newDate) && !isNil(endDate) && newDate > endDate) {
      newDate = endDate;
    }

    applyDates(event, newDate, endDate);

    if (!isNil(newDate)) {
      var _endDateRef$current;

      (_endDateRef$current = endDateRef.current) === null || _endDateRef$current === void 0 ? void 0 : _endDateRef$current.focus();
    }
  });
  const handleEndDateInputValueChange = useEventCallback(event => {
    // @ts-ignore
    const newCharacter = event.nativeEvent.data; // If the new character is not a digit, we don't want to do anything since the new character will be removed by the mask input.
    // The digit is test with a regex because this is how our mask input third party is doing it and we want to be consistant.

    if (/\d/.test(newCharacter)) {
      if (isNilOrEmpty(event.target.value)) {
        var _startDateRef$current2;

        (_startDateRef$current2 = startDateRef.current) === null || _startDateRef$current2 === void 0 ? void 0 : _startDateRef$current2.focus();
      }
    }
  });
  const handleEndDateChange = useEventCallback((event, newDate) => {
    if (!isNil(newDate) && !isNil(startDate) && newDate < startDate) {
      newDate = startDate;
    }

    applyDates(event, startDate, newDate);
  });
  const handleSelectPreset = useEventCallback((event, keys) => {
    const index = parseInt(keys[0]);
    const preset = presets[index];

    if (!isNil(preset)) {
      applyDates(event, preset.startDate, preset.endDate);
    }
  });
  const handleClearDates = useEventCallback(event => {
    applyDates(event, null, null);
    startDateRef === null || startDateRef === void 0 ? void 0 : startDateRef.current.focus();
  });
  const handleContainerKeyDown = useEventCallback(event => {
    if (event.key === Keys.esc) {
      event.preventDefault();
      handleClearDates(event);
    }
  });
  const handleEndDateKeyDown = useEventCallback(event => {
    if (event.key === Keys.backspace) {
      var _endDateRef$current2;

      if (isNilOrEmpty((_endDateRef$current2 = endDateRef.current) === null || _endDateRef$current2 === void 0 ? void 0 : _endDateRef$current2.value)) {
        var _startDateRef$current3;

        (_startDateRef$current3 = startDateRef.current) === null || _startDateRef$current3 === void 0 ? void 0 : _startDateRef$current3.focus();
      }
    }
  });
  const focusWithinProps = useFocusWithin({
    onFocus: useEventCallback(event => {
      if (!isNil(onFocus)) {
        onFocus(event);
      }

      setHasFocus(true);
    }),
    onBlur: useEventCallback(event => {
      if (!isNil(onBlur)) {
        onBlur(event);
      }

      setHasFocus(false);
    })
  });
  const hasValue = !isNil(startDate) || !isNil(endDate);

  const inputMarkup = /*#__PURE__*/_jsxs(Box, { ...mergeProps(focusWithinProps, {
      onKeyDown: handleContainerKeyDown,
      className: cssModule("o-ui-date-range-input", validationState, fluid && "fluid", disabled && "disabled", readOnly && "readonly", active && "active", hasFocus && "focus", hover && "hover"),
      role: !isInField ? "group" : undefined
    }),
    children: [/*#__PURE__*/_jsx(CalendarIcon, {
      className: "o-ui-date-range-input-icon"
    }), /*#__PURE__*/_jsx(DateInput, {
      value: startDate,
      placeholder: placeholder,
      required: required,
      validationState: validationState,
      min: min,
      max: max,
      onDateChange: handleStartDateChange,
      autoFocus: autoFocus,
      disabled: disabled,
      readOnly: readOnly,
      name: !isNil(name) ? `${name}-start-date` : undefined,
      ref: startDateRef
    }), /*#__PURE__*/_jsx(Divider, {
      orientation: "vertical",
      className: "o-ui-date-range-input-divider"
    }), /*#__PURE__*/_jsx(DateInput, {
      value: endDate,
      placeholder: placeholder,
      required: required,
      validationState: validationState,
      min: min,
      max: max,
      onChange: handleEndDateInputValueChange,
      onDateChange: handleEndDateChange,
      onKeyDown: handleEndDateKeyDown,
      disabled: disabled,
      readOnly: readOnly,
      name: !isNil(name) ? `${name}-end-date` : undefined,
      tabIndex: hasFocus ? 0 : -1,
      ref: endDateRef
    }), hasValue && !disabled && !readOnly && /*#__PURE__*/_jsx(CrossButton, {
      onClick: handleClearDates,
      size: "xs",
      condensed: true,
      className: "o-ui-date-range-input-clear-button",
      "aria-label": "Clear dates"
    })]
  });

  const presetsMarkup = !isNil(presets) && /*#__PURE__*/_jsxs(MenuTrigger, {
    children: [/*#__PURE__*/_jsx(IconButton, {
      shape: "rounded",
      color: "secondary",
      condensed: true,
      "aria-label": "Date presets",
      children: /*#__PURE__*/_jsx(VerticalDotsIcon, {})
    }), /*#__PURE__*/_jsx(Menu, {
      onSelectionChange: handleSelectPreset,
      children: presets.map((x, index) =>
      /*#__PURE__*/
      // eslint-disable-next-line react/no-array-index-key
      _jsx(Item, {
        children: x.text
      }, index.toString()))
    })]
  });

  const container = !isNil(presets) ? /*#__PURE__*/_jsxs(Box, {
    className: "o-ui-date-range-input-group",
    role: "presentation",
    children: [inputMarkup, presetsMarkup]
  }) : inputMarkup; // HACK: Returning the augmented element in a fragment to comply with react-docgen.

  return /*#__PURE__*/_jsx(_Fragment, {
    children: augmentElement(container, mergeProps(rest, {
      as,
      ref: containerRef
    }))
  });
}
export const DateRangeInput = forwardRef((props, ref) => /*#__PURE__*/_jsx(InnerDateRangeInput, { ...props,
  forwardedRef: ref
}));
DateRangeInput.displayName = "DateRangeInput";